@@ -3,28 +3,32 @@ function(data, D = NULL, k=1,  msplit=10, Rep=250, cut.in=15){
 
  n <- dim(data)[1]
  all <- 1:n
- if(is.null(D) | (class(D) != "dist"))
-	D <- rrp.dist(data,  msplit=msplit, Rep=Rep,cut.in=cut.in, check.bal=FALSE, 
+ if(is.null(D) | (!any(class(D) == "XPtr")))
+	d <- rrp.dist(data,  msplit=msplit, Rep=Rep,cut.in=cut.in, check.bal=FALSE, 
 		plot=FALSE)
- if(attr(D,"method") != "RRP")
-	stop("`dist' object `D' must have attribute `method' = `RRP'")
-	
+
  miss.obs <- which(apply(data, 1, function(x) length(which(is.na(x)))>0 ) == TRUE)
- 
+ D <- XPtrToDist(d) 
  D[which(D==1)] <- NA
- M <- as.matrix(D)
  
  y <- data
  comp.obs <- all[-miss.obs]
 
+ f <- function(x) {tmp <- order(x); tmp[1:min(k,length(tmp))]}
+ nn <- applyXPtr(d, miss.obs, comp.obs, f)
+
+ g <- function(xx) {tmp <- order(xx); 
+   tmp <- tmp[1:min(k,length(tmp))]; xx[tmp]}
+ ww <- applyXPtr(d, miss.obs, comp.obs, g)
+
  for(i in 1:length(miss.obs)){
-  tmp <- as.integer(order(M[miss.obs[i], comp.obs]))
-  tmp <- tmp[1:min(k,length(tmp))]
+  tmp <- nn[[i]] 
+  wt <- ww[[i]]
 
   v.idx <- which(is.na(data[miss.obs[i],]))	
   for(v in v.idx){
     if(!is.factor(data[[v]])){
-	  y[miss.obs[i],v] <- weighted.mean(data[comp.obs[tmp],v], w=1-M[miss.obs[i],comp.obs[tmp]], na.rm=TRUE)
+	  y[miss.obs[i],v] <- weighted.mean(data[comp.obs[tmp],v], w=1-wt, na.rm=TRUE)
 	} else {
 	
 	  if(length(tmp)>1){
